@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2005 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -29,20 +29,21 @@
 
     /* the library version */
 #ifndef PTH_VERSION_STR
-#define PTH_VERSION_STR "1.4.1 (27-Jan-2002)"
+#define PTH_VERSION_STR "2.0.4 (18-Feb-2005)"
 #endif
 #ifndef PTH_VERSION_HEX
-#define PTH_VERSION_HEX 0x104201
+#define PTH_VERSION_HEX 0x200204
 #endif
 #ifndef PTH_VERSION
 #define PTH_VERSION PTH_VERSION_HEX
 #endif
 
     /* essential headers */
-#include <sys/types.h>     /* for ssize_t, off_t */
-#include <sys/time.h>      /* for struct timeval */
-#include <sys/socket.h>    /* for sockaddr       */
-#include <sys/signal.h>    /* for sigset_t       */
+#include <sys/types.h>     /* for ssize_t, off_t  */
+#include <time.h>          /* for struct timespec */
+#include <sys/time.h>      /* for struct timeval  */
+#include <sys/socket.h>    /* for sockaddr        */
+#include <sys/signal.h>    /* for sigset_t        */
 #include <sys/select.h>
 
     /* fallbacks for essential typedefs */
@@ -53,11 +54,12 @@
 /* typedef socklen_t socklen_t; */
 /* typedef int off_t; */
 /* typedef int sig_atomic_t; */
-typedef int nfds_t;
+/* typedef nfds_t nfds_t; */
 #endif /* !_PTHREAD_PRIVATE */
 
     /* extra structure definitions */
 struct timeval;
+struct timespec;
 
     /* essential values */
 #ifndef FALSE
@@ -83,6 +85,13 @@ struct timeval;
 #else
 #define BEGIN_DECLARATION /*nop*/
 #define END_DECLARATION   /*nop*/
+#endif
+
+    /* check if the user requests a bigger FD_SETSIZE than we can handle */
+#if defined(FD_SETSIZE)
+#if FD_SETSIZE > 1024
+#error "FD_SETSIZE is larger than what GNU Pth can handle."
+#endif
 #endif
 
 BEGIN_DECLARATION
@@ -115,6 +124,7 @@ BEGIN_DECLARATION
                                        PTH_CTRL_GETTHREADS_SUSPENDED|\
                                        PTH_CTRL_GETTHREADS_DEAD)
 #define PTH_CTRL_DUMPSTATE            _BIT(10)
+#define PTH_CTRL_FAVOURNEW            _BIT(11)
 
     /* the time value structure */
 typedef struct timeval pth_time_t;
@@ -149,6 +159,7 @@ enum {
     PTH_ATTR_CANCEL_STATE,   /* RW [unsigned int]      thread cancellation state         */
     PTH_ATTR_STACK_SIZE,     /* RW [unsigned int]      stack size                        */
     PTH_ATTR_STACK_ADDR,     /* RW [char *]            stack lower address               */
+    PTH_ATTR_DISPATCHES,     /* RO [int]               total number of thread dispatches */
     PTH_ATTR_TIME_SPAWN,     /* RO [pth_time_t]        time thread was spawned           */
     PTH_ATTR_TIME_LAST,      /* RO [pth_time_t]        time thread was last dispatched   */
     PTH_ATTR_TIME_RAN,       /* RO [pth_time_t]        time thread was running           */
@@ -199,6 +210,13 @@ enum { PTH_FREE_THIS, PTH_FREE_ALL };
 #define PTH_WALK_NEXT                _BIT(1)
 #define PTH_WALK_PREV                _BIT(2)
 
+    /* event status codes */
+typedef enum {
+    PTH_STATUS_PENDING,
+    PTH_STATUS_OCCURRED,
+    PTH_STATUS_FAILED
+} pth_status_t;
+
     /* the key type and init value */
 typedef int pth_key_t;
 #define PTH_KEY_INIT (-1)
@@ -304,6 +322,10 @@ struct pth_barrier_st { /* not hidden to avoid destructor */
     pth_mutex_t   br_mutex;
 };
 
+    /* the user-space context structure */
+typedef struct pth_uctx_st *pth_uctx_t;
+struct pth_uctx_st;
+
     /* filedescriptor blocking modes */
 enum {
     PTH_FDMODE_ERROR = -1,
@@ -317,7 +339,27 @@ enum {
 #define PTH_FAKE_POLL 0
 #if !(PTH_FAKE_POLL)
 /* use vendor poll(2) environment */
+#ifndef _XOPEN_SOURCE
+#define _XOPEN_SOURCE
+#define _XOPEN_SOURCE_set
+#endif
 #include <poll.h>
+#ifdef _XOPEN_SOURCE_set
+#undef _XOPEN_SOURCE_set
+#undef _XOPEN_SOURCE
+#endif
+#ifndef POLLRDNORM
+#define POLLRDNORM POLLIN
+#endif
+#ifndef POLLRDBAND
+#define POLLRDBAND POLLIN
+#endif
+#ifndef POLLWRNORM
+#define POLLWRNORM POLLOUT
+#endif
+#ifndef POLLWRBAND
+#define POLLWRBAND POLLOUT
+#endif
 #ifndef INFTIM
 #define INFTIM (-1)
 #endif
@@ -420,7 +462,7 @@ extern int            pth_event_extract(pth_event_t ev, ...);
 extern pth_event_t    pth_event_concat(pth_event_t, ...);
 extern pth_event_t    pth_event_isolate(pth_event_t);
 extern pth_event_t    pth_event_walk(pth_event_t, unsigned int);
-extern int            pth_event_occurred(pth_event_t);
+extern pth_status_t   pth_event_status(pth_event_t);
 extern int            pth_event_free(pth_event_t, int);
 
     /* key-based storage functions */
@@ -460,6 +502,12 @@ extern int            pth_cond_notify(pth_cond_t *, int);
 extern int            pth_barrier_init(pth_barrier_t *, int);
 extern int            pth_barrier_reach(pth_barrier_t *);
 
+    /* user-space context functions */
+extern int            pth_uctx_create(pth_uctx_t *);
+extern int            pth_uctx_make(pth_uctx_t, char *, size_t, const sigset_t *, void (*)(void *), void *, pth_uctx_t);
+extern int            pth_uctx_switch(pth_uctx_t, pth_uctx_t);
+extern int            pth_uctx_destroy(pth_uctx_t);
+
     /* extension functions */
 extern Sfdisc_t      *pth_sfiodisc(void);
 
@@ -479,6 +527,7 @@ extern ssize_t        pth_recvfrom_ev(int, void *, size_t, int, struct sockaddr
 extern ssize_t        pth_sendto_ev(int, const void *, size_t, int, const struct sockaddr *, socklen_t, pth_event_t);
 
     /* standard replacement functions */
+extern int            pth_nanosleep(const struct timespec *, struct timespec *);
 extern int            pth_usleep(unsigned int);
 extern unsigned int   pth_sleep(unsigned int);
 extern pid_t          pth_waitpid(pid_t, int *, int);
@@ -488,6 +537,7 @@ extern int            pth_sigwait(const sigset_t *, int *);
 extern int            pth_connect(int, const struct sockaddr *, socklen_t);
 extern int            pth_accept(int, struct sockaddr *, socklen_t *);
 extern int            pth_select(int, fd_set *, fd_set *, fd_set *, struct timeval *);
+extern int            pth_pselect(int, fd_set *, fd_set *, fd_set *, const struct timespec *, const sigset_t *);
 extern int            pth_poll(struct pollfd *, nfds_t, int);
 extern ssize_t        pth_read(int, void *, size_t);
 extern ssize_t        pth_write(int, const void *, size_t);
@@ -507,10 +557,13 @@ END_DECLARATION
 #define fork          pth_fork
 #define waitpid       pth_waitpid
 #define system        pth_system
+#define nanosleep     pth_nanosleep
+#define usleep        pth_usleep
 #define sleep         pth_sleep
 #define sigprocmask   pth_sigmask
 #define sigwait       pth_sigwait
 #define select        pth_select
+#define pselect       pth_pselect
 #define poll          pth_poll
 #define connect       pth_connect
 #define accept        pth_accept
@@ -526,5 +579,10 @@ END_DECLARATION
 #define pwrite        pth_pwrite
 #endif
 
+    /* backward compatibility (Pth < 1.5.0) */
+#define pth_event_occurred(ev) \
+    (   pth_event_status(ev) == PTH_STATUS_OCCURRED \
+     || pth_event_status(ev) == PTH_STATUS_FAILED   )
+
 #endif /* _PTH_H_ */
 
